@@ -8,11 +8,14 @@
     :copyright: Conceptual Vision Consulting LLC 2018-2019, see AUTHORS for more details.
     :license: MIT, see LICENSE for more details.
 """
+from abc import ABC
 
-class IRegisterable():
+
+class IRegisterable(ABC):
     """
     Interface to perform on-demand registrations.
     """
+
     def register(self):
         """
         Perform required registration steps.
